# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flake8_docstrings_complete']

package_data = \
{'': ['*']}

install_requires = \
['flake8>=5']

entry_points = \
{'flake8.extension': ['DCO = flake8_docstrings_complete:Plugin']}

setup_kwargs = {
    'name': 'flake8-docstrings-complete',
    'version': '1.0.4',
    'description': 'A linter that checks docstrings are complete',
    'long_description': '# flake8-docstrings-complete\n\nLinter that checks docstrings of functions, methods and classes. It should be\nused in conjunction with `pydocstyle` (or `flake8-docstrings`) as the linter\nassumes that the docstrings already pass `pydocstyle` checks. This\n[blog post](https://jdkandersson.com/2023/01/07/writing-great-docstrings-in-python/)\ndiscusses how to write great docstrings and the motivation for this linter!\n\n## Getting Started\n\n```shell\npython -m venv venv\nsource ./venv/bin/activate\npip install flake8 flake8-docstrings flake8-docstrings-complete\nflake8 source.py\n```\n\nOn the following code where the `foo` function has the `bar` and `baz`\narguments where the `baz` argument is missing from the `Args` section in the\ndocstring:\n\n```Python\n# source.py\ndef foo(bar, baz):\n    """Perform foo action on bar.\n\n    Args:\n        bar: The value to perform the foo action on.\n    """\n```\n\nThis will produce warnings such as:\n\n```shell\nflake8 test_source.py\nsource.py:2:14: DCO023 "baz" argument should be described in the docstring, more information: https://github.com/jdkandersson/flake8-docstrings-complete#fix-dco023\n```\n\nThis can be resolved by adding the `baz` argument to the `Args` section:\n\n```Python\n# source.py\ndef foo(bar, baz):\n    """Perform foo action on bar.\n\n    Args:\n        bar: The value to perform the foo action on.\n        baz: The modifier to the foo action.\n    """\n```\n\n`flake8-docstrings-complete` adds the following checks to complement\n`pydocstyle`:\n\n1. If a function/ method has arguments, that the arguments section is included.\n2. If a function/ method has arguments, that all function/ method arguments are\n   in the argument section.\n3. If an arguments section is in the function/ method docstring, the argument\n   section contains no arguments the function/ method doesn\'t have.\n4. If a function/ method has a return statement with a value, the return value\n   section is included.\n5. If a function/ method has a yield statement with a value, the yield value\n   section is included.\n6. If a function/ method raises an exception, the raises section is included\n   with a description for each exception that is raised.\n7. If a class has public attributes, that the attributes section is included.\n8. If a class has public attributes, that all public attributes are in the\n   attributes section.\n9. If an attributes section is in the class docstring, the attributes section\n   contains no attributes the class doesn\'t have.\n10. Any of the sections being checked are not present multiple times.\n\nNote:\n\n- `self` and `cls` are not counted as arguments.\n- `test_.*` methods are skipped in `test_.*\\.py` files (function and file names\n  are configurable).\n- functions with a `@fixture` et al dectorator in `conftest.py` and\n  `test_.*\\.py` files are skipped (function and fixture file names are\n  configurable)\n\n## Configuration\n\nThe plugin adds the following configurations to `flake8`:\n\n- `--docstrings-complete-test-filename-pattern`: The filename pattern for test\n  files. Defaults to `test_.*\\.py`.\n- `--docstrings-complete-test-function-pattern`: The function name pattern for\n  test functions. Defaults to `test_.*`.\n- `--docstrings-complete-fixture-filename-pattern`: The filename pattern for\n  fixture files. Defaults to `conftest\\.py`.\n- `--docstrings-complete-fixture-decorator-pattern`: The decorator name pattern\n  for fixture functions. Defaults to `(^|\\.)fixture$`.\n\n## Rules\n\nA few rules have been defined to allow for selective suppression:\n\n- `DCO010`: docstring missing on a function/ method/ class.\n- `DCO020`: function/ method has one or more arguments and the docstring does\n  not have an arguments section.\n- `DCO021`: function/ method with no arguments and the docstring has an\n  arguments section.\n- `DCO022`: function/ method with one or more arguments and the docstring has\n  multiple arguments sections.\n- `DCO023`: function/ method has one or more arguments not described in the\n  docstring.\n- `DCO024`: function/ method has one or more arguments described in the\n  docstring which are not arguments of the function/ method.\n- `DCO030`: function/ method that returns a value does not have the returns\n  section in the docstring.\n- `DCO031`: function/ method that does not return a value has the returns\n  section in the docstring.\n- `DCO032`: function/ method that returns a value and the docstring has\n  multiple returns sections.\n- `DCO040`: function/ method that yields a value does not have the yields\n  section in the docstring.\n- `DCO041`: function/ method that does not yield a value has the yields\n  section in the docstring.\n- `DCO042`: function/ method that yields a value and the docstring has\n  multiple yields sections.\n- `DCO050`: function/ method raises one or more exceptions and the docstring\n  does not have a raises section.\n- `DCO051`: function/ method that raises no exceptions and the docstring has a\n  raises section.\n- `DCO052`: function/ method that raises one or more exceptions and the\n  docstring has multiple raises sections.\n- `DCO053`: function/ method that raises one or more exceptions where one or\n  more of the exceptions is not described in the docstring.\n- `DCO054`: function/ method has one or more exceptions described in the\n  docstring which are not raised in the function/ method.\n- `DCO055`: function/ method that has a raise without an exception has an empty\n  raises section in the docstring.\n- `DCO060`: class has one or more public attributes and the docstring does not\n  have an attributes section.\n- `DCO061`: class with no attributes and the docstring has an attributes\n  section.\n- `DCO062`: class with one or more attributes and the docstring has multiple\n  attributes sections.\n- `DCO063`: class has one or more public attributes not described in the\n  docstring.\n- `DCO064`: class has one or more attributes described in the docstring which\n  are not attributes of the class.\n\n### Fix DCO010\n\nThis linting rule is triggered by a function/ method/ class without a\ndocstring. For example:\n\n```Python\ndef foo():\n    pass\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action."""\n        pass\n\nclass FooClass:\n    """Perform foo action."""\n    def foo(self):\n        pass\n```\n\nThis example can be fixed by adding a docstring:\n\n```Python\ndef foo():\n    """Perform foo action."""\n\nclass FooClass:\n    """Perform foo action."""\n    def foo(self):\n        """Perform foo action."""\n```\n\n### Fix DCO020\n\nThis linting rule is triggered by a function/ method that has one or more\narguments with a docstring that does not have an arguments section. For\nexample:\n\n```Python\ndef foo(bar):\n    """Perform foo action."""\n\nclass FooClass:\n    def foo(self, bar):\n        """Perform foo action."""\n```\n\nThese examples can be fixed by adding the arguments section and describing all\narguments in the arguments section:\n\n```Python\ndef foo(bar):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n    """\n\ndef foo(bar):\n    """Perform foo action.\n\n    Arguments:\n        bar: the value to perform the foo action on.\n    """\n\ndef foo(bar):\n    """Perform foo action.\n\n    Parameters:\n        bar: the value to perform the foo action on.\n    """\n\nclass FooClass:\n    def foo(self, bar):\n        """Perform foo action.\n\n        Args:\n            bar: the value to perform the foo action on.\n        """\n```\n\n### Fix DCO021\n\nThis linting rule is triggered by a function/ method that has no arguments with\na docstring that has an arguments section. For example:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n    """\n\ndef foo():\n    """Perform foo action.\n\n    Arguments:\n        bar: the value to perform the foo action on.\n    """\n\ndef foo():\n    """Perform foo action.\n\n    Parameters:\n        bar: the value to perform the foo action on.\n    """\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Args:\n            bar: the value to perform the foo action on.\n        """\n```\n\nThese examples can be fixed by removing the arguments section:\n\n```Python\ndef foo():\n    """Perform foo action."""\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action."""\n```\n\n### Fix DCO022\n\nThis linting rule is triggered by a function/ method that has one or more\narguments and a docstring that has multiple arguments sections. For example:\n\n```Python\ndef foo(bar):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n\n    Args:\n        bar: the value to perform the foo action on.\n    """\n\ndef foo(bar):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n\n    Arguments:\n        bar: the value to perform the foo action on.\n\n    Parameters:\n        bar: the value to perform the foo action on.\n    """\n\nclass FooClass:\n    def foo(self, bar):\n        """Perform foo action.\n\n        Args:\n            bar: the value to perform the foo action on.\n\n        Args:\n            bar: the value to perform the foo action on.\n        """\n```\n\nThese examples can be fixed by removing the additional arguments sections:\n\n```Python\ndef foo(bar):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n    """\n\ndef foo(bar):\n    """Perform foo action.\n\n    Arguments:\n        bar: the value to perform the foo action on.\n    """\n\ndef foo(bar):\n    """Perform foo action.\n\n    Parameters:\n        bar: the value to perform the foo action on.\n    """\n\nclass FooClass:\n    def foo(self, bar):\n        """Perform foo action.\n\n        Args:\n            bar: the value to perform the foo action on.\n        """\n```\n\n### Fix DCO023\n\nThis linting rule is triggered by a function/ method that has one or more\narguments where one or more of those arguments is not described in the\ndocstring. For example:\n\n```Python\ndef foo(bar):\n    """Perform foo action.\n\n    Args:\n    """\n\ndef foo(bar, baz):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n    """\n\nclass FooClass:\n    def foo(self, bar):\n        """Perform foo action.\n\n        Args:\n        """\n```\n\nThese examples can be fixed by adding the missing arguments to the arguments\nsection:\n\n```Python\ndef foo(bar):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n    """\n\ndef foo(bar, baz):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n        baz: the modifier to the foo action.\n    """\n\nclass FooClass:\n    def foo(self, bar):\n        """Perform foo action.\n\n        Args:\n            bar: the value to perform the foo action on.\n        """\n```\n\n### Fix DCO024\n\nThis linting rule is triggered by a function/ method that has one or more\narguments and a docstring that describes one or more arguments where on or more\nof the described arguments are not arguments of the function/ method. For\nexample:\n\n```Python\ndef foo(bar):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n        baz: the modifier to the foo action.\n    """\n\nclass FooClass:\n    def foo(self, bar):\n        """Perform foo action.\n\n        Args:\n            bar: the value to perform the foo action on.\n            baz: the modifier to the foo action.\n        """\n```\n\nThese examples can be fixed by removing the arguments the function/ method\ndoesn\'t have from the docstring:\n\n```Python\ndef foo(bar):\n    """Perform foo action.\n\n    Args:\n        bar: the value to perform the foo action on.\n    """\n\nclass FooClass:\n    def foo(self, bar):\n        """Perform foo action.\n\n        Args:\n            bar: the value to perform the foo action on.\n        """\n```\n\n### Fix DCO030\n\nThis linting rule is triggered by a function/ method that has at least one\nreturn statement with a value and does not have a returns section in the\ndocstring. For example:\n\n```Python\ndef foo():\n    """Return bar."""\n    return "bar"\n\nclass FooClass:\n    def foo(self):\n        """Return bar."""\n        return "bar"\n```\n\nThese examples can be fixed by adding the returns section:\n\n```Python\ndef foo():\n    """Return bar.\n\n    Return:\n        bar.\n    """\n    return "bar"\n\ndef foo():\n    """Return bar.\n\n    Returns:\n        bar.\n    """\n    return "bar"\n\nclass FooClass:\n    def foo(self):\n        """Return bar.\n\n        Returns:\n            bar.\n        """\n        return "bar"\n```\n\n### Fix DCO031\n\nThis linting rule is triggered by a function/ method that has no return\nstatement with a value and has a returns section in the\ndocstring. For example:\n\n```Python\ndef foo():\n    """Return bar.\n\n    Returns:\n        bar.\n    """\n    pass\n\nclass FooClass:\n    def foo(self):\n        """Return bar.\n\n        Returns:\n            bar.\n        """\n        pass\n```\n\nThese examples can be fixed by removing the returns section:\n\n```Python\ndef foo():\n    """Take foo action."""\n    pass\n\nclass FooClass:\n    def foo(self):\n        """Take foo action."""\n        pass\n```\n\n### Fix DCO032\n\nThis linting rule is triggered by a function/ method that returns a value and\nhas a docstring that has multiple returns sections. For example:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Returns:\n        bar.\n\n    Returns:\n        bar.\n    """\n    return "bar"\n\ndef foo():\n    """Perform foo action.\n\n    Returns:\n        bar.\n\n    Return:\n        bar.\n    """\n    return "bar"\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Returns:\n            bar.\n\n        Returns:\n            bar.\n        """\n        return "bar"\n```\n\nThese examples can be fixed by removing the additional returns sections:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Returns:\n        bar.\n    """\n    return "bar"\n\ndef foo():\n    """Perform foo action.\n\n    Returns:\n        bar.\n    """\n    return "bar"\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Returns:\n            bar.\n        """\n        return "bar"\n```\n\n### Fix DCO040\n\nThis linting rule is triggered by a function/ method that has at least one\nyield statement with a value or a yield from statement and does not have a\nyields section in the docstring. For example:\n\n```Python\ndef foo():\n    """Yield bar."""\n    yield "bar"\n\ndef foo():\n    """Yield bar."""\n    yield from ("bar",)\n\nclass FooClass:\n    def foo(self):\n        """Yield bar."""\n        yield "bar"\n```\n\nThese examples can be fixed by adding the yields section:\n\n```Python\ndef foo():\n    """Yield bar.\n\n    Yield:\n        bar.\n    """\n    yield "bar"\n\ndef foo():\n    """Yield bar.\n\n    Yields:\n        bar.\n    """\n    yield "bar"\n\ndef foo():\n    """Yield bar.\n\n    Yields:\n        bar.\n    """\n    yield from ("bar",)\n\nclass FooClass:\n    def foo(self):\n        """Yield bar.\n\n        Yields:\n            bar.\n        """\n        yield "bar"\n```\n\n### Fix DCO041\n\nThis linting rule is triggered by a function/ method that has no yield\nstatement with a value nor a yield from statement and has a yields section\nin the docstring. For example:\n\n```Python\ndef foo():\n    """Yield bar.\n\n    Yields:\n        bar.\n    """\n    pass\n\nclass FooClass:\n    def foo(self):\n        """Yield bar.\n\n        Yields:\n            bar.\n        """\n        pass\n```\n\nThese examples can be fixed by:\n\n```Python\ndef foo():\n    """Take foo action."""\n    pass\n\nclass FooClass:\n    def foo(self):\n        """Take foo action."""\n        pass\n```\n\n### Fix DCO042\n\nThis linting rule is triggered by a function/ method that yields a value and\nhas a docstring that has multiple yields sections. For example:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Yields:\n        bar.\n\n    Yields:\n        bar.\n    """\n    yield "bar"\n\ndef foo():\n    """Perform foo action.\n\n    Yields:\n        bar.\n\n    Yields:\n        bar.\n    """\n    yield from ("bar",)\n\ndef foo():\n    """Perform foo action.\n\n    Yields:\n        bar.\n\n    Yield:\n        bar.\n    """\n    yield "bar"\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Yields:\n            bar.\n\n        Yields:\n            bar.\n        """\n        yield "bar"\n```\n\nThese examples can be fixed by removing the additional yields sections:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Yields:\n        bar.\n    """\n    yield "bar"\n\ndef foo():\n    """Perform foo action.\n\n    Yields:\n        bar.\n    """\n    yield from ("bar",)\n\ndef foo():\n    """Perform foo action.\n\n    Yields:\n        bar.\n    """\n    yield "bar"\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Yields:\n            bar.\n        """\n        yield "bar"\n```\n\n### Fix DCO050\n\nThis linting rule is triggered by a function/ method that raises one or more\nexceptions and a docstring that does not have a raises section. For example:\n\n```Python\ndef foo():\n    """Perform foo action."""\n    raise BarError\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action."""\n        raise BarError\n```\n\nThese examples can be fixed by adding the raises section and describing all\nraised exceptions in it:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n\ndef foo():\n    """Perform foo action.\n\n    Raise:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n        """\n        raise BarError\n```\n\n### Fix DCO051\n\nThis linting rule is triggered by a function/ method that raises no exceptions\nwith a docstring that has a raises section. For example:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n\ndef foo():\n    """Perform foo action.\n\n    Raise:\n        BarError: the value to perform the foo action on was wrong.\n    """\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n        """\n```\n\nThese examples can be fixed by removing the raises section:\n\n```Python\ndef foo():\n    """Perform foo action."""\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action."""\n```\n\n### Fix DCO052\n\nThis linting rule is triggered by a function/ method that raises one or more\nexceptions with a docstring that has multiple raises sections. For example:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n\n    Raise:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n        """\n        raise BarError\n```\n\nThese examples can be fixed by removing the additional raises sections:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n\ndef foo():\n    """Perform foo action.\n\n    Raise:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n        """\n        raise BarError\n```\n\n### Fix DCO053\n\nThis linting rule is triggered by a function/ method that raises one or more\nexceptions where one or more of those exceptions is not described in the\ndocstring. For example:\n\n```Python\ndef foo():\n    """Perform foo action."""\n    raise BarError\n\ndef foo(bar, baz):\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n    raise BazError\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action."""\n        raise BarError\n```\n\nThese examples can be fixed by describing the additional exceptions in the\ndocstring:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n\ndef foo(bar, baz):\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n        BazError: the alternate value to perform the foo action on was wrong.\n    """\n    raise BarError\n    raise BazError\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n        """\n        raise BarError\n```\n\n### Fix DCO054\n\nThis linting rule is triggered by a function/ method that raises one or more\nexceptions and a docstring that describes one or more exceptions where on or\nmore of the described exceptions are not raised by the function/ method. For\nexample:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n        BazError: the alternate value to perform the foo action on was wrong.\n    """\n    raise BarError\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n            BazError: the alternate value to perform the foo action on was wrong.\n        """\n        raise BarError\n```\n\nThese examples can be fixed by removing the exception that is not raised from\nthe docstring:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    raise BarError\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n        """\n        raise BarError\n```\n\n### Fix DCO055\n\nThis linting rule is triggered by a function/ method that has a `raise`\nstatement without an exception (typically re-raising exceptions) and the raises\nsection is not included or is empty. For example:\n\n```Python\ndef foo():\n    """Perform foo action."""\n    try:\n        bar()\n    except BarError:\n        raise\n\ndef foo():\n    """Perform foo action.\n\n    Raises:\n    """\n    try:\n        bar()\n    except BarError:\n        raise\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action."""\n        try:\n            bar()\n        except BarError:\n            raise\n```\n\nThese examples can be fixed by describing at least one exception in the raises\nsection:\n\n```Python\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    try:\n        bar()\n    except BarError:\n        raise\n\ndef foo():\n    """Perform foo action.\n\n    Raises:\n        BarError: the value to perform the foo action on was wrong.\n    """\n    try:\n        bar()\n    except BarError:\n        raise\n\nclass FooClass:\n    def foo(self):\n        """Perform foo action.\n\n        Raises:\n            BarError: the value to perform the foo action on was wrong.\n        """\n        try:\n            bar()\n        except BarError:\n            raise\n```\n\n### Fix DCO060\n\nThis linting rule is triggered by a class that has one or more public\nattributes with a docstring that does not have an attributes section. For\nexample:\n\n```Python\nclass FooClass:\n    """Perform foo action."""\n    bar = "bar"\n\nclass FooClass:\n    """Perform foo action."""\n\n    def __init__(self):\n        self.bar = "bar"\n\nclass FooClass:\n    """Perform foo action."""\n\n    def bar(self):\n        self.baz = "baz"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n    baz = "baz"\n```\n\nThese examples can be fixed by adding the attributes section and describing all\nattributes in the attributes section:\n\n```Python\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attributes:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attributes:\n        bar: The value to perform the foo action on.\n    """\n\n    def __init__(self):\n        self.bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attributes:\n        baz: The value to perform the foo action on.\n    """\n\n    def bar(self):\n        self.baz = "baz"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n        baz: The alternate value to perform the foo action on.\n    """\n    bar = "bar"\n    baz = "baz"\n```\n\n### Fix DCO061\n\nThis linting rule is triggered by a class that has no attributes with a\ndocstring that has an attributes section. For example:\n\n```Python\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n\nclass FooClass:\n    """Perform foo action.\n\n    Attributes:\n        bar: The value to perform the foo action on.\n    """\n\nclass FooClass:\n    """Perform foo action.\n\n    Attributes:\n    """\n\n    def __init__(self):\n        self._bar = "bar"\n```\n\nThese examples can be fixed by removing the attributes section:\n\n```Python\nclass FooClass:\n    """Perform foo action."""\n\nclass FooClass:\n    """Perform foo action."""\n\n    def __init__(self):\n        self._bar = "bar"\n```\n\n### Fix DCO062\n\nThis linting rule is triggered by a class that has one or more attributes and\na docstring that has multiple attributes sections. For example:\n\n```Python\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n\n    Attributes:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n```\n\nThese examples can be fixed by removing the additional attributes sections:\n\n```Python\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attributes:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n```\n\n### Fix DCO063\n\nThis linting rule is triggered by a class that has one or more public\nattributes where one or more of those public attributes is not described in the\ndocstring. For example:\n\n```Python\nclass FooClass:\n    """Perform foo action."""\n    bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n    """\n\n    def __init__(self):\n        self.bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n    """\n\n    @property\n    def bar(self):\n        return "bar"\n\nclass FooClass:\n    """Perform foo action."""\n\n    def bar(self):\n        self.baz = "baz"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n    baz = "baz"\n```\n\nThese examples can be fixed by adding the missing attributes to the attributes\nsection:\n\n```Python\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attributes:\n        bar: The value to perform the foo action on.\n    """\n\n    def __init__(self):\n        self.bar = "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n\n    @property\n    def bar(self):\n        return "bar"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attributes:\n        baz: The value to perform the foo action on.\n    """\n\n    def bar(self):\n        self.baz = "baz"\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n        baz: The alternate value to perform the foo action on.\n    """\n    bar = "bar"\n    baz = "baz"\n```\n\n### Fix DCO064\n\nThis linting rule is triggered by a class that has one or more attributes and a\ndocstring that describes one or more attributes where on or more\nof the described attributes are not attributes of the class. For example:\n\n```Python\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        _bar: The value to perform the foo action on.\n    """\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n        baz: The alternate value to perform the foo action on.\n    """\n    bar = "bar"\n```\n\nThese examples can be fixed by removing the attributes the class doesn\'t have\nfrom the docstring:\n\n```Python\nclass FooClass:\n    """Perform foo action."""\n\nclass FooClass:\n    """Perform foo action."""\n\nclass FooClass:\n    """Perform foo action.\n\n    Attrs:\n        bar: The value to perform the foo action on.\n    """\n    bar = "bar"\n```\n\n## Docstring Examples\n\nExamples of function/ method and class docstrings are:\n\n```Python\ndef foo(bar):\n    """Perform the foo actions on bar.\n\n    Args:\n        bar: the value to perform the foo actions on.\n\n    Returns:\n        bar after applying to foo action to it.\n\n    Yields:\n        All the foo actions that have been performed.\n\n    Raises:\n        FooError: an error occurred whilst performing the foo action.\n    """\n\nclass Foo:\n    """Foo object.\n\n    Attrs:\n        bar: the value to perform the foo actions on.\n    """\n\n    def __init__(self, bar):\n        """Construct.\n\n        Args:\n            bar: the value to perform the foo actions on.\n        """\n        self.bar = bar\n```\n\n## Sections\n\nThere are several alternative names for each of the sections which are captured\ncase-insensitive:\n\n- arguments: `Args`, `Arguments`, `Parameters`\n- return value: `Return`, `Returns`\n- yield value: `Yield`, `Yields`\n- raise: `Raises`\n- attributes: `Attrs`, `Attributes`\n\nSection information is extracted using the following algorithm:\n\n1. Look for a line that has zero or more whitespace characters, followed by a\n   section name (non-case-sensistive) followed by a colon.\n2. Look for any sub-sections on a line which starts with zero or more\n   whitespace characters followed by a word, optionally followed by whitespace\n   and any characters within round brackets followed by a colon.\n3. The section ends if any line with zero or more whitespace characters is\n   encountered or the end of the docstring is reached.\n\n## Future Ideas:\n\n- Check that argument, exceptions and attributes have non-empty description.\n- Check that arguments, exceptions and attributes are only documented once.\n- Check that arguments, exceptions and attributes have meaningful descriptions.\n- Check other other PEP257 conventions\n',
    'author': 'David Andersson',
    'author_email': 'david@jdkandersson.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)
